import java.awt.*;

//   newest

     // BART line; i.e. Richmond-Colma, Colma-Richmond, etc.
class Bline
{
  SchedReader s;
  TransitVehicle transitVehicles[];
  int trainSize;
  int npixels=0 ;	        // number of pixels
  int x[] ;		// x values
  int y[] ;		// y values
  int slp[] ;		// codes for slope
  int nstations=0 ;	// number of stations

  int lineid;
  int nruns ;    		// number of runs
  int dir_codes[] ;		// direction of each segment in this line
  int segoffs[] ;		// direction of offset
  int offsets[] ;		// amount of offset
  int at_station[] ;		// locations of each station on line

// temporary
  int start_ms ;		// starting time in milliseconds since midnight
  int startDiffsMs[] ;		// time between each run
  int workingStDiffs[] ;

  Bline(int tsize, SchedReader schedReader, Bsegment bartSegments[], int lineid)
  {
    int j;
    int nsegs ;    		// number of segments in this line
    int seg_codes[] ;		// codes for which segments in this line

    this.lineid = lineid;
    trainSize = tsize;
    s = schedReader;
      //  number of segments in line
    nsegs = s.iget() ;
    seg_codes = new int[nsegs] ;
    dir_codes = new int[nsegs] ;
    segoffs = new int[nsegs] ;
    for (j = 0 ; j < nsegs ; j++)
    {
        //  segment codes for this line
      seg_codes[j] = s.iget() ;
        //  directions of segments for this line
      dir_codes[j] = s.iget() ;
        // this is not general; specifically for this system
        // offset for this segment
      segoffs[j] = s.iget() ;
        //  find number of stations in line
      nstations += bartSegments[seg_codes[j]].nstations ;
    }
    at_station = new int[nstations] ;
    merge_segs(bartSegments, nsegs, seg_codes);
  }

  void readTimes()
  {
    int total = 0;
    int tmp;
    int val;
    int i, j;
    boolean copyrow[] ;	//  whether to copy last line in schedule
      //  string holding current integer to be put into
      //  buffer holding times between stations
    String strv ;
    int incrTime;

    nruns = s.iget() ;

    startDiffsMs = new int[nruns] ;
    workingStDiffs = new int[nruns] ;

    transitVehicles = new TransitVehicle[nruns];
    for (i = 0; i < nruns; i++)
      transitVehicles[i] = new TransitVehicle(this, nstations);

      // read start time;
      // convert minutes to milliseconds as read in
    start_ms = Clock.MS_IN_MINUTE * s.iget() ;
    while (total < nruns)
    {
        //  run length
      tmp = s.iget() ;
        //  value
      val = s.iget() ;
      for (j = tmp ; j > 0 ; j--)
         transitVehicles[total++].startStation = val ; 
    }
    total = 0 ;
    while (total < nruns)
    {
      tmp = s.iget() ;
      val = s.iget() ;
      for (j = tmp ; j > 0 ; j--)
        transitVehicles[total++].endStation = val ; 
    }

    copyrow = new boolean[nruns] ;
    copyrow[0] = false ;
    startDiffsMs[0] = 0;
    total = 1 ;
      //  for all runs in line
    while (total < nruns)
    {
      tmp = s.iget() ;
        // not a run-length, value pair
      if (tmp > 0)
      {
          // convert minutes to milliseconds
        startDiffsMs[total] = Clock.MS_IN_MINUTE * (tmp-1);
        copyrow[total] = false ;
        total++ ;
      }
      else
      {
        val = s.iget() ;
        for (j = -tmp ; j > 0 ; j--, total++)
        {
          startDiffsMs[total] = Clock.MS_IN_MINUTE * (val-1);
            //  copy previous row
          copyrow[total] = true ;
        }
      }
    }
      // read wait times between each station in minutes
      // for all runs in line
    for (total = 0 ; total < nruns ; total++)
    {
      incrTime = 0;
          //  if not copying previous row (run)
      if (!copyrow[total])
      {
        val = s.iget() ;
        if (val > 1)
        {
          transitVehicles[total].timetable[1] = val-1;
          incrTime += val-1;
        }
        else if (val < -1)
        {
          transitVehicles[total].timetable[1] = (-val)-1;
          incrTime += (-val)-1;
        }
        else	// the starting station is not the first station in the line
        {
                // this can lead to 0 waits between some stations
          transitVehicles[total].timetable[1] = (float)0.0 ;
        }
          // for all stations except the first
        for (j = 2 ; j < nstations ; j++)
        {
                //  if some times between stations are greater than 1 digit
                //  i.e., times are separated by spaces
          if (val < 0)
          {
            tmp = s.iget()-1;
            if (tmp != 0)
            {
              transitVehicles[total].timetable[j] = (float)tmp - (float)0.5; 
              incrTime += tmp;
            }
            else
              transitVehicles[total].timetable[j] = (float)0.0; 
          }
          else	//  all times one digit; not separated by spaces
          {
            strv = s.cget(j);
            tmp = Integer.parseInt(strv);
            if (tmp != 1)
            {
                //  take into account wait at station
              transitVehicles[total].timetable[j] = (float)tmp - (float)1.5 ;
              incrTime += tmp - 1;
            }
            else	//  the starting station is not the first station
              transitVehicles[total].timetable[j] = (float)0.0 ;
          }
        }
        if (val >= 0)
          s.advance();
        transitVehicles[total].totalTime = incrTime * Clock.MS_IN_MINUTE;
      }
      else	//  copy previous row (run)
      {
        for (j = 1 ; j < nstations ; j++)
          transitVehicles[total].timetable[j] = transitVehicles[total-1].timetable[j];
        transitVehicles[total].totalTime = transitVehicles[total-1].totalTime;
      }
    }
    calcPixelWaits();
  }


  void calcPixelWaits()
  {
    int j, k;
    int pixsBetweenStations;

    for (j = 0 ; j < nruns; j++)
    {
      for (k = transitVehicles[j].startStation+1 ; k <= transitVehicles[j].endStation; k++)
      {
        pixsBetweenStations = (at_station[k] - at_station[k-1]);
        transitVehicles[j].timetable[k] *= (float)Clock.MS_IN_MINUTE/((float)pixsBetweenStations);
      }
    }
  }


    // generates each line by merging segments
  void merge_segs(Bsegment bsegs[], int nsegs, int seg_codes[])
  {
    int i, j, k ;
    Bsegment curr_seg ;		//  current segment being merged
    int loc ;		//  current location on line
    int pix_endpt ;		//  one of ends of segment
    int st_loc ;		//  location of station
    int station_ctr ;	//  number of stations so far
    int nSegPixels;
	
      // find number of pixels in line
    for (j = 0 ; j < nsegs ; j++)
    {
      npixels += bsegs[seg_codes[j]].npixels ;
    }
    npixels -= nsegs - 1 ;
    x = new int[npixels] ;
    y = new int[npixels] ;
    slp = new int[npixels] ;
    offsets = new int[npixels] ;
    loc = 0 ;
    station_ctr = 0 ;
      // for all segments making up this line
    for (j = 0 ; j < nsegs ; j++)
    {
      curr_seg = bsegs[seg_codes[j]] ;
      nSegPixels = curr_seg.npixels ;
        //  go through segment in forward order
      if (dir_codes[j] == 1)
      {
        pix_endpt = (j == 0) ? 0 : 1 ;
        st_loc = 0 ;
          //  for all pixels in segment
        for (k = pix_endpt ; k < nSegPixels ; k++, loc++)
        {
          x[loc] = curr_seg.x[k] ;
          y[loc] = curr_seg.y[k] ;
          slp[loc] = curr_seg.slp[k] ;
              // kludge for Mac-Arthur section
          if ((k > 2) && (k < (nSegPixels - 24)))
          {
            if (segoffs[j] > 0)
              offsets[loc] = trainSize + 1 ;
            else if (segoffs[j] < 0)
              offsets[loc] = -(trainSize + 1) ;
          }
          else
            offsets[loc] = 0 ;
              //  if at station location, save it
          if ((st_loc < curr_seg.nstations) &&
              (curr_seg.x[k] == curr_seg.stx[st_loc]) &&
              (curr_seg.y[k] == curr_seg.sty[st_loc]))
          {
            at_station[station_ctr] = loc ;
            station_ctr++ ;
            st_loc++ ;
          }
        }
      }
      else	//  go through segment in reverse order
      {
        pix_endpt = (j == 0) ? nSegPixels-1 : nSegPixels-2 ;
        st_loc = curr_seg.nstations-1 ;
          //  for all pixels in this segment
        for (k = pix_endpt ; k >= 0 ; k--, loc++)
        {
          x[loc] = curr_seg.x[k] ;
          y[loc] = curr_seg.y[k] ;
            //  take into account reverse order when calculating slope code
          slp[loc] = (curr_seg.slp[k] & 4) | (~(curr_seg.slp[k] & 3) & 3) ;
             //   kludge for MacArthur section
          if ((k > 2) && (k < (nSegPixels - 24)))
          {
            if (segoffs[j] > 0)
              offsets[loc] = trainSize + 1 ;
            else if (segoffs[j] < 0)
              offsets[loc] = -(trainSize + 1) ;
          }
          else
            offsets[loc] = 0 ;
              //  if at station location, save it
          if ((st_loc >= 0) && (curr_seg.x[k] == curr_seg.stx[st_loc]) &&
			    (curr_seg.y[k] == curr_seg.sty[st_loc]))
          {
            at_station[station_ctr] = loc ;
            st_loc-- ;
            station_ctr++ ;
          }
        }
      }
    }
  }
}
