import java.awt.Color;
import java.awt.*;

class TransitVehicle
{
  Bline bline;
  Color color;
  int pixelsToDraw;
  int loc;
  float leftOverMs;
  float pixelWaitMs;
  int whichStation;
  boolean trainRemoved;

  int lineid;

    //  these are set in Bline in readTimes
  int startStation;
  int endStation;
  float timetable[];
  int totalTime;


  TransitVehicle(Bline b, int nstations)
  {
    this.lineid = b.lineid;
    this.bline = b;
    color = TransitInfo.colors[(b.lineid)/2];
    timetable = new float[nstations];
  }


  void reset(int overshotMs)
  {
    trainRemoved = false;
    leftOverMs = (float) overshotMs;
    whichStation = startStation + 1  ;
    loc = bline.at_station[startStation] + 6 ;
    pixelsToDraw = 7 ;
      //  wait between each pixel
    pixelWaitMs = timetable[whichStation];
  }


  boolean beenRemoved()
  {
    return(trainRemoved);
  }


  void update(int diffMs, int trainSize, boolean catchUp)
  {
    float sumMs;

      //  time overshot on last go-through +
    sumMs = leftOverMs; 
      //  update train position if time consumed less than amount
      //  between this update period and last
    while ((int)sumMs <= diffMs)
    {
        //  increase amount of time consumed
      sumMs += pixelWaitMs; 
      //  by advancing train position one pixel
        //  latest train position
      loc++ ;
        //  increase number of pixels to draw
      pixelsToDraw++ ;
        // if train at station
      if (loc == bline.at_station[whichStation])
      {
          //  if not at last station
        if (whichStation < endStation)
        {
          whichStation++ ;
          pixelWaitMs = timetable[whichStation];
        }
          //  increase amount of time consumed by
          //  time spent at station
        sumMs += (float) Clock.STATION_SLEEP ;
        if (!catchUp)
          break;
      }
        // if train run ended
      else if (loc > (bline.at_station[endStation] - 1))
      {
        pixelsToDraw = -1;
        break;
      }
    }
        //  amount overshot
    leftOverMs = sumMs - (float)diffMs ;
    if ((catchUp) && (pixelsToDraw > trainSize))
      pixelsToDraw = trainSize;
  }


  void paint(Graphics g, Image img, SchedManager imgCanv, int trainSize)
  {
    int i, j ;
    int train_loc ;		// location of part of current train
    int xinc, yinc ;		// places train on one or other side of track
    int xpos, ypos ;		// current position redrawing

      //  if train run ended
    if (!trainRemoved)
    {
      if (pixelsToDraw < 0)
      {
        pixelsToDraw = 0;
        train_loc = bline.at_station[endStation] - trainSize;
          //  erase the train that has ended its run
        for (i = 0 ; i <= trainSize ; i++, train_loc++)
          erase_trailing_edge(g,img,train_loc,imgCanv,trainSize) ;
        trainRemoved = true;
      }
        //  if train needs to be updated
      else if (pixelsToDraw > 0)
      {
        for (train_loc = (loc - pixelsToDraw) + 1; train_loc <= loc; train_loc++)
        {
            g.setColor(color);
                    // draw leading edge in color of train
            xpos = bline.x[train_loc] ;
            ypos = bline.y[train_loc] ;
              // if horizontal
            if (bline.slp[train_loc] > 3)
            {
                // only horizontal trains handle offsets at this time
              xinc = ((bline.slp[train_loc] % 2) == 1) ? -(trainSize+1) : 2 ;
              xinc += bline.offsets[train_loc] ;
              xpos = bline.x[train_loc] + xinc ;
              ypos = bline.y[train_loc] ;
              g.drawLine(xpos,ypos,xpos+(trainSize-1),ypos) ;
            }
            else	// vertical
            {
              yinc = ((bline.slp[train_loc] & 2) == 2) ? 2 : -(trainSize+1) ;
              xpos = bline.x[train_loc] ;
              ypos = bline.y[train_loc] + yinc ;
              g.drawLine(xpos,ypos,xpos,ypos+(trainSize-1)) ;
            }
              // if not at first station
            if (train_loc >= (bline.at_station[startStation] + (trainSize-1)))
            {
              erase_trailing_edge(g,img,train_loc-(trainSize-1),imgCanv, trainSize) ;
            }
            pixelsToDraw = 0 ;
        }
      }
    }
  }


    // erase trailing edge of one train
  void erase_trailing_edge(Graphics g, Image img, int trailing_edge,
                           SchedManager imgCanv, int trainSize)
  {
    Graphics subg ;
    int xpos, ypos ;
    int xinc, yinc ;
    int drawid;

            // draw trailing line in background
        if (bline.slp[trailing_edge] > 3)	// horizontal
        {
		// find position of trailing line
            xinc = ((bline.slp[trailing_edge] % 2) == 1) ? -(trainSize+1) : 2 ;
            xinc += bline.offsets[trailing_edge] ;
            xpos = bline.x[trailing_edge] + xinc ;
            ypos = bline.y[trailing_edge] ;
              //  create graphics context for clipping amount of image drawn
              //  expensive call
            subg = g.create(xpos,ypos,trainSize,1) ;
        }
        else		// vertical
        {
		// find position of trailing line
            yinc = ((bline.slp[trailing_edge] & 2) == 2) ? 2 : -(trainSize+1) ;
            xpos = bline.x[trailing_edge] ;
            ypos = bline.y[trailing_edge] + yinc ;
            subg = g.create(xpos,ypos,1,trainSize) ;
        }
	    //  very expensive call
        subg.drawImage(img,-xpos,-ypos,imgCanv) ;
	    //  expensive call
        subg.dispose() ;
  }
}
